import json
from functools import lru_cache

import pygments
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.template import RequestContext, loader
from django.urls import reverse
from django.views.decorators.csrf import csrf_exempt
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name

from paste.forms import PasteForm
from paste.models import Paste
from webtools import settings


@csrf_exempt
def index(request):
    """Displays form."""
    if request.method == "GET":
        return render(request, "paste/index.html")
    paste = Paste()
    paste.choose_slug()
    if not request.FILES:
        return HttpResponse(json.dumps({"error": "Please provide a file."}, indent=4))
    (filename, the_file), *_ = request.FILES.items()
    form = PasteForm(
        {
            "filename": filename,
            "content": the_file.read().decode(),
        },
        instance=paste,
    )
    if not form.is_valid():
        return HttpResponse(json.dumps(form.errors, indent=4))
    form.save()
    location = request.build_absolute_uri(
        reverse("short_paste", kwargs={"slug": paste.slug})
    )
    return HttpResponseRedirect(
        location, content=location + "\n", content_type="text/plain"
    )


def show(request, slug):
    """Display paste."""

    paste = get_object_or_404(Paste, slug=slug)
    paste.incr_viewcount()

    if "html" in request.headers["accept"]:
        return HttpResponse(pygmentize(paste.filename, paste.content))
    else:
        return HttpResponse(paste.content, content_type="text/plain")


@lru_cache(1024)
def pygmentize(filename, filecontents):
    try:
        lexer = get_lexer_by_name(filename.split(".")[-1])
    except pygments.util.ClassNotFound:
        lexer = get_lexer_by_name(settings.PASTE["default_language"])
    formatter = HtmlFormatter(style="emacs")
    data = {}
    data["filename"] = filename
    data["highlighted"] = pygments.highlight(filecontents, lexer, formatter)
    return loader.render_to_string("paste/show-pygments.html", data)
